unit FTPParams;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, IniFiles, Buttons, ToolWin, ComCtrls,
  OverbyteIcsWndControl, OverbyteIcsFtpCli ;

type
  TfrmFTPParams = class(TForm)
    Panel1: TPanel;
    Panel6: TPanel;
    txtTemplateTitle: TStaticText;
    Panel3: TPanel;
    TBXToolbar1: TToolbar;
    btnCancel: TSpeedButton;
    btnSave: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    edtHost: TEdit;
    edtFolder: TEdit;
    edtUsername: TEdit;
    edtPassword: TEdit;
    edtPort: TEdit;
    ftpSend: TFtpClient;
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure edtHostChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Function NewTempFile(aFile: String): String;
  end;

var
  frmFTPParams: TfrmFTPParams;

implementation

uses Main;

{$R *.dfm}

procedure TfrmFTPParams.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

Function TfrmFTPParams.NewTempFile(aFile: String): String;
var
  TmpPath : String;
  TmpPathLen : Integer;

begin
  TmpPathLen := 512;
  SetLength(TmpPath, TmpPathLen + 1);
  TmpPathLen := GetTempPath(TmpPathLen, PChar(TmpPath));
  If TmpPathLen = 0 then
    TmpPath := 'c:\'
  else
    SetLength(TmpPath, TmpPathLen );
  If TmpPath[length(TmpPath)] <> '\' then
    TmpPath := TmpPath + '\';
  Result := Format('%s~ICC-%d\', [TmpPath, GetTickCount]);
  If Not DirectoryExists(Result) then
    CreateDir(Result);

  Result := Result + aFile + '.html';
end;

procedure TfrmFTPParams.btnSaveClick(Sender: TObject);
var
  FileName            : String;
  Ini                 : TIniFile;
begin
{ get info from ini file}
  Ini := TIniFile.Create( ChangeFileExt( Application.ExeName, '.INI' ) );
  ftpSend.HostName := edtHost.Text;
  ftpSend.Port := edtPort.Text;
  ftpSend.UserName := edtUsername.Text;
  ftpSend.PassWord := edtPassword.Text;
  Try
    ftpSend.Connect;
  Except
    On E: Exception do
      MessageDlg(E.Message, mtError, [mbOK], 0);
  end;

  If ftpSend.Connected then begin
{ save in ini file}
    Ini.WriteString('FTP','Host',edtHost.Text);
    Ini.WriteString('FTP','Port',edtPort.Text);
    Ini.WriteString('FTP','Folder',edtFolder.Text);
    Ini.WriteString('FTP','Username',edtUsername.Text);
    Ini.WriteString('FTP','Password',edtPassword.Text);
    If edtFolder.Text <> '' then begin
      ftpSend.HostDirName := edtFolder.Text;
      Try
        ftpSend.Cwd;
      Except
        On E: Exception do
          MessageDlg(E.Message, mtError, [mbOK], 0);
      end;
    end;

    FileName := NewTempFile(frmMain.theProject.Title);
    frmMain.theProject.Article.SaveToFile(FileName);
    ftpSend.LocalFileName := FileName;
    ftpSend.HostFileName := ExtractFileName(FileName);
    ftpSend.HostName := edtHost.Text;
    ftpSend.UserName := edtUsername.Text;
    ftpSend.PassWord := edtPassword.Text;
    Try
      If ftpSend.Put then
        MessageDlg('Article successfully uploaded to FTP server!', mtInformation, [mbOK], 0)
      else
        MessageDlg('Could not upload the article to FTP server! Check you parameters and try again!', mtWarning, [mbOK], 0);
    Except
      On E: Exception do
        MessageDlg(E.Message, mtError, [mbOK], 0);
    end;

    If FileExists(FileName) then
      DeleteFile(FileName);
    If DirectoryExists(ExtractFilePath(FileName)) then
      RemoveDir(ExtractFilePath(FileName));
  end
  else
    MessageDlg('Could not connect to FTP server! Check you parameters and try again!', mtError, [mbAbort], 0);

  Ini.Free;
  ModalResult := mrOK;
end;

procedure TfrmFTPParams.edtHostChange(Sender: TObject);
begin
  btnSave.Enabled := (Trim(edtHost.Text) <> '') And (Trim(edtUsername.Text) <> '') And
                     (Trim(edtPort.Text) <> '') And (Trim(edtPassword.Text) <> '');
end;

procedure TfrmFTPParams.FormShow(Sender: TObject);
var
  Ini                 : TIniFile;
begin
{ get info from ini file}
  Ini := TIniFile.Create( ChangeFileExt( Application.ExeName, '.INI' ) );
  if (FileExists(Ini.FileName)) then begin
      edtHost.Text     :=  Ini.ReadString( 'FTP', 'Host', '' );
      edtPort.Text     :=  Ini.ReadString( 'FTP', 'Port', '' );
      edtFolder.Text   :=  Ini.ReadString( 'FTP', 'Folder', '' );
      edtUsername.Text :=  Ini.ReadString( 'FTP', 'Username', '' );
      edtPassword.Text :=  Ini.ReadString( 'FTP', 'Password', '' );
  end;
  edtHost.OnChange(Sender);
  Ini.Free;
end;

end.
