unit Find;

{ LOG FROM SOURCESAFE --
   $Log:: /Delphi/MSHTML Editor/Find.pas                                      $
 * 
 * 4     9/10/01 3:04p Danr
 * Added the ability to limit the find to a selected area. 
 * 
 * If no selection is made then we check the entire document from the top.
 *
 * 3     9/06/01 2:31p Danr
 * Form is always on top.
 *
   $NoKeywords: $
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Mshtml_Edit, MSHTML_TLB;

type
  TfrmFind = class(TForm)
    labFind: TLabel;
    edtFind: TEdit;
    grpOption: TGroupBox;
    cmdCancel: TButton;
    cmdNext: TButton;
    chkWholeWord: TCheckBox;
    chkMatchCase: TCheckBox;
    tmrKick: TTimer;
    procedure edtFindKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmdCancelClick(Sender: TObject);
    procedure chkWholeWordClick(Sender: TObject);
    procedure chkMatchCaseClick(Sender: TObject);
    procedure cmdNextClick(Sender: TObject);
    procedure tmrKickTimer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmFind: TfrmFind;
  bFirst,
  bWholeWord,   {Find whole word only}
  bMatchCase,   {Case sensitive}
  bForward:     {Search from top to bottom}
    boolean;
  FRangeSave,   {Save original range info}
  FRangeTemp:   {Temporary range}
    IHTMLTxtRange;
  FiLen:        {Length to search or selection length}
    integer;

implementation

{$R *.DFM}


{Initialize the text range object}
procedure TfrmFind.FormShow(Sender: TObject);
begin
  tmrKick.Enabled := True;
end;


{The real initialize... started by timer event in show}
procedure TfrmFind.tmrKickTimer(Sender: TObject);
begin
  tmrKick.Enabled         := False;
  frmFind.BringToFront;
  edtFind.Text            := '';
  cmdNext.Enabled         := False;
  chkWholeWord.Checked    := True;
  chkMatchCase.Checked    := False;
  bWholeWord := chkWholeWord.Checked;
  bMatchCase := chkMatchCase.Checked;
  bFirst := True;
  with frmMSHTMLEdit do
  begin
    FRangeSave := FRange.duplicate;
    FRangeTemp := Frange.duplicate;
    if FbHaveSelect then
      FiLen := length(FRangeSave.text)
    else
      FiLen := length(FsHTMLImage);
  end;
  edtFind.SetFocus;
end;


{Turn on the find button once we have something to look for}
procedure TfrmFind.edtFindKeyPress(Sender: TObject; var Key: Char);
begin
  if (length(edtFind.Text) > 0) then
  begin
    cmdNext.Enabled := True;
    with frmMSHTMLEdit do
    begin
      if not FbHaveSelect then
        FRange.moveStart('textedit', -1);
      FRange.collapse(True);
    end;
  end
  else
    cmdNext.Enabled := False;
  if Key = #13 then
  begin
    Key := #0;
    if (cmdNext.Enabled) then
    begin
      with frmMSHTMLEdit do
      begin
        FRangeSave := FRange.duplicate;
        FRangeTemp := Frange.duplicate;
        if FbHaveSelect then
          FiLen := length(FRangeSave.text)
        else
          FiLen := length(FsHTMLImage);
      end;
      cmdNextClick(self);
    end;
  end;
end;


{Clear the text range object}
procedure TfrmFind.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  edtFind.Text            := '';
  cmdNext.Enabled         := False;
  chkWholeWord.Checked    := True;
  chkMatchCase.Checked    := False;
  bWholeWord := chkWholeWord.Checked;
  bMatchCase := chkMatchCase.Checked;
  with frmMSHTMLEdit do
  begin
    {Restore the original range}
    FRange := FRangeSave.duplicate;
    FRangeSave := nil;
    FRangeTemp := nil;
    {Restore the selection if needed}
    if FbHaveSelect then
    begin
      FRange.select;
      FRange.scrollIntoView(True);
      FbHaveSelect := False;
    end;
  end;
end;


{Close}
procedure TfrmFind.cmdCancelClick(Sender: TObject);
begin
  Close;
end;


{Full or partial word search}
procedure TfrmFind.chkWholeWordClick(Sender: TObject);
begin
  bWholeWord := chkWholeWord.Checked;
end;


{Case sensitivity}
procedure TfrmFind.chkMatchCaseClick(Sender: TObject);
begin
  bMatchCase := chkMatchCase.Checked;
end;


{Find next...}
procedure TfrmFind.cmdNextClick(Sender: TObject);
var
  bFound:   boolean;
  iFlag:    integer;
begin
  iFlag := 0;
  if bWholeWord then
    iFlag := iFlag + 2;
  if bMatchCase then
    iFlag := iFlag + 4;
  with frmMSHTMLEdit do
  begin

    {Make sure we stay in a selection (if we have one)}
    FRange.collapse(bFirst);
    bFound := FRange.findText(edtFind.Text, FiLen, iFlag);
    FRangeTemp := Frange.Duplicate;
    if bFound then
      if (FRangeSave.inRange(FRangeTemp)) then
        bFound := True
      else
        bFound := False;                  {End of selection}

    {If we found it and it's in range, show it}
    if (bFound) then
    begin
      {If we found it, select it and make sure it's visible}
      FRange.select;
      FRange.scrollIntoView(True);
    end
    else
    begin
      {Otherwise just dump out an error}
      ShowMessage('"' + edtFind.Text + '" was not found!');
      cmdNext.Enabled := False;
      edtFind.SetFocus;
    end;
  end;
  if bFirst then
    bFirst := False;
end;


end.
