unit Replace;

{ LOG FROM SOURCESAFE --
   $Log:: /Delphi/MSHTML Editor/Replace.pas                                   $
 * 
 * 5     9/10/01 3:06p Danr
 * Added the ability to limit the find/replace to a selected area. 
 * 
 * If no selection is made then we check the entire document from the top.
 * 
 * Made the final message from replace all a bit smarter in both what it did
 * and the grammar used.
 * 
 * 4     9/06/01 2:33p Danr
 * Added always on top.
 *
   $NoKeywords: $
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, MSHTML_TLB;

type
  TfrmReplace = class(TForm)
    labFind: TLabel;
    edtFind: TEdit;
    tmrKick: TTimer;
    grpOption: TGroupBox;
    chkWholeWord: TCheckBox;
    chkMatchCase: TCheckBox;
    cmdNext: TButton;
    cmdCancel: TButton;
    cmdReplace: TButton;
    cmdReplaceAll: TButton;
    edtReplace: TEdit;
    labReplace: TLabel;
    procedure FormShow(Sender: TObject);
    procedure tmrKickTimer(Sender: TObject);
    procedure edtFindKeyPress(Sender: TObject; var Key: Char);
    procedure edtReplaceKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmdNextClick(Sender: TObject);
    procedure chkWholeWordClick(Sender: TObject);
    procedure chkMatchCaseClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure cmdReplaceClick(Sender: TObject);
    procedure cmdReplaceAllClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
  CRLF = #13 + #10;

var
  frmReplace: TfrmReplace;
  bFirst,
  bWholeWord,   {Find whole word only}
  bMatchCase,   {Case sensitive}
  bForward:     {Search from top to bottom}
    boolean;
  FRangeSave,   {Save original range info}
  FRangeTemp:   {Temporary range}
    IHTMLTxtRange;
  FiLen:        {Length to search or selection length}
    integer;

implementation

uses mshtml_edit;

{$R *.DFM}


{Initialize the text range object}
procedure TfrmReplace.FormShow(Sender: TObject);
begin
  tmrKick.Enabled := True;
end;


{The real initialize... started by timer event in show}
procedure TfrmReplace.tmrKickTimer(Sender: TObject);
begin
  tmrKick.Enabled := False;
  frmReplace.BringToFront;
  edtFind.Text            := '';
  cmdNext.Enabled         := False;
  cmdReplace.Enabled      := False;
  cmdReplaceAll.Enabled   := False;
  chkWholeWord.Checked    := True;
  chkMatchCase.Checked    := False;
  bWholeWord := chkWholeWord.Checked;
  bMatchCase := chkMatchCase.Checked;
  bFirst := True;
  with frmMSHTMLEdit do
  begin
    FRangeSave := FRange.duplicate;
    FRangeTemp := Frange.duplicate;
    if FbHaveSelect then
      FiLen := length(FRangeSave.text)
    else
      FiLen := length(FsHTMLImage);
  end;
  edtFind.SetFocus;
end;


{Clear the text range object}
procedure TfrmReplace.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  edtFind.Text            := '';
  edtReplace.Text         := '';
  cmdNext.Enabled         := False;
  cmdReplace.Enabled      := False;
  cmdReplaceAll.Enabled   := False;
  chkWholeWord.Checked    := True;
  chkMatchCase.Checked    := False;
  bWholeWord := chkWholeWord.Checked;
  bMatchCase := chkMatchCase.Checked;
  with frmMSHTMLEdit do
  begin
    {Restore the original range}
    FRange := FRangeSave.duplicate;
    FRangeSave := nil;
    FRangeTemp := nil;
    {Restore the selection if needed}
    if FbHaveSelect then
    begin
      FRange.select;
      FRange.scrollIntoView(True);
      FbHaveSelect := False;
    end;
  end;
end;


{Turn on the find button once we have something to look for and replace with}
procedure TfrmReplace.edtFindKeyPress(Sender: TObject; var Key: Char);
begin
  if ((length(edtFind.Text) > 0) and
      (length(edtReplace.Text) >0)) then
  begin
    cmdNext.Enabled := True;
    with frmMSHTMLEdit do
    begin
      if not FbHaveSelect then
        FRange.moveStart('textedit', -1);
      FRange.collapse(True);
    end;
  end
  else
    cmdNext.Enabled := False;
  if Key = #13 then
  begin
    Key := #0;
    if (cmdNext.Enabled) then
    begin
      with frmMSHTMLEdit do
      begin
        FRangeSave := FRange.duplicate;
        FRangeTemp := Frange.duplicate;
        if FbHaveSelect then
          FiLen := length(FRangeSave.text)
        else
          FiLen := length(FsHTMLImage);
      end;
      cmdNextClick(self);
    end;
  end;
end;


{Turn on the find button once we have something to look for and replace with}
procedure TfrmReplace.edtReplaceKeyPress(Sender: TObject; var Key: Char);
begin
  if ((length(edtFind.Text) > 0) and
      (length(edtReplace.Text) >0)) then
  begin
    cmdNext.Enabled := True;
    with frmMSHTMLEdit do
    begin
      if not FbHaveSelect then
        FRange.moveStart('textedit', -1);
      FRange.collapse(True);
    end;
  end
  else
    cmdNext.Enabled := False;
  if Key = #13 then
  begin
    Key := #0;
    if (cmdNext.Enabled) then
    begin
      with frmMSHTMLEdit do
      begin
        FRangeSave := FRange.duplicate;
        FRangeTemp := Frange.duplicate;
        if FbHaveSelect then
          FiLen := length(FRangeSave.text)
        else
          FiLen := length(FsHTMLImage);
      end;
      cmdNextClick(self);
    end;
  end;
end;


{Find next...}
procedure TfrmReplace.cmdNextClick(Sender: TObject);
var
  bFound:   boolean;
  //iLen:     integer;
  iFlag:    integer;
begin
  iFlag := 0;
  if bWholeWord then
    iFlag := iFlag + 2;
  if bMatchCase then
    iFlag := iFlag + 4;
  with frmMSHTMLEdit do
  begin
    //iLen := length(FsHTMLImage);
    //FRange.collapse(bFirst);

    {Make sure we stay in a selection (if we have one)}
    FRange.collapse(bFirst);
    bFound := FRange.findText(edtFind.Text, FiLen, iFlag);
    FRangeTemp := Frange.Duplicate;
    if bFound then
      if (FRangeSave.inRange(FRangeTemp)) then
        bFound := True
      else
        bFound := False;                  {End of selection}

    if (bFound) then
    begin
      {If we found it, select it and make sure it's visible}
      FRange.select;
      FRange.scrollIntoView(True);
      cmdReplace.Enabled    := True;
      cmdReplaceAll.Enabled := True;
    end
    else
    begin
      {Otherwise just dump out an error}
      ShowMessage('"' + edtFind.Text + '" was not found!');
      cmdNext.Enabled       := False;
      cmdReplace.Enabled    := False;
      cmdReplaceAll.Enabled := False;
      edtFind.SetFocus;
    end;
  end;
  if bFirst then
    bFirst := False;
end;


{Replace the current selection, find the next one...}
procedure TfrmReplace.cmdReplaceClick(Sender: TObject);
begin
  with frmMSHTMLEdit do
  begin
    FRange.Set_text(edtReplace.Text);
  end;
  cmdNextClick(self);
end;


{Find and replace all... report finds}
procedure TfrmReplace.cmdReplaceAllClick(Sender: TObject);
var
  bFound,
  bDone:    boolean;
  iFlag,
  iDid:     integer;
begin
  iFlag := 0;
  iDid  := 0;
  if bWholeWord then
    iFlag := iFlag + 2;
  if bMatchCase then
    iFlag := iFlag + 4;
  with frmMSHTMLEdit do
  begin
    bDone := False;
    FRange.Set_text(edtReplace.Text);
    inc(iDid);
    if bFirst then
      bFirst := False;
    while (not bDone) do
    begin
      {Make sure we stay in a selection (if we have one)}
      FRange.collapse(bFirst);
      bFound := FRange.findText(edtFind.Text, FiLen, iFlag);
      FRangeTemp := Frange.Duplicate;
      if bFound then
        if (FRangeSave.inRange(FRangeTemp)) then
          bFound := True
        else
          bFound := False;                  {End of selection}

      if (bFound) then
      begin
        {If we found it, select it and make sure it's visible}
        FRange.select;
        FRange.scrollIntoView(True);
        FRange.Set_text(edtReplace.Text);
        inc(iDid);
      end
      else
      begin
        {Otherwise just dump out an error and bail out of the loop}
        if iDid < 1 then
          ShowMessage('"' + edtFind.Text + '" was not found!')
        else if idid = 1 then
          ShowMessage('"' + edtFind.Text + '" was replaced with' + CRLF +
                      '"' + edtReplace.Text + '".' + CRLF + CRLF +
                      'Replaced ' + IntToStr(iDid) + ' word.')
        else
          ShowMessage('"' + edtFind.Text + '" was replaced with' + CRLF +
                      '"' + edtReplace.Text + '".' + CRLF + CRLF +
                      'Replaced ' + IntToStr(iDid) + ' words.');
        bDone := True;
      end;
    end;
  end;
  cmdNext.Enabled       := False;
  cmdReplace.Enabled    := False;
  cmdReplaceAll.Enabled := False;
  edtFind.SetFocus;
end;


{Full or partial word search}
procedure TfrmReplace.chkWholeWordClick(Sender: TObject);
begin
  bWholeWord := chkWholeWord.Checked;
end;


{Case sensitivity}
procedure TfrmReplace.chkMatchCaseClick(Sender: TObject);
begin
  bMatchCase := chkMatchCase.Checked;
end;


{Close}
procedure TfrmReplace.cmdCancelClick(Sender: TObject);
begin
  Close;
end;


end.
